import json

def save_to_json(data, filename):
    with open(filename, 'w') as json_file:
        json.dump(data, json_file, indent=4)

def load_from_json(filename):
    with open(filename, 'r') as json_file:
        data = json.load(json_file)
    return data



import unittest
from unittest.mock import Mock
from pyonemap.core import OneMap

class TestOneMap(unittest.TestCase):
    def setUp(self):
        self.onemap = OneMap()

    def test_search(self):
        mock_response = {
            "results": [
                {
                    "SEARCHVAL": "5 Havelock Road",
                    "BLK_NO": "5",
                    "ROAD_NAME": "Havelock Road",
                    "POSTAL": "169610",
                    "X": "29068.100058874996",
                    "Y": "29357.549260427598",
                    "LATITUDE": "1.288576327",
                    "LONGITUDE": "103.83245770099999"
                }
            ]
        }

        self.onemap._make_request = Mock(return_value=mock_response)

        result = self.onemap.search("5 Havelock Road")
        self.assertEqual(result, mock_response["results"])

    def test_search(self):
        mock_response = {
            "results": {'GeocodeInfo': [{'BUILDINGNAME': 'NEW TOWN PRIMARY SCHOOL', 'BLOCK': '300', 'ROAD': 'TANGLIN HALT ROAD', 'POSTALCODE': '148812', 'XCOORD': '24303.3101027', 'YCOORD': '31333.3389857', 'LATITUDE': '1.2996418818103135', 'LONGITUDE': '103.80010216304007'}]}
        }

        self.onemap._make_request = Mock(return_value=mock_response)

        result = self.onemap.revGeoCodeXy(24291.97788882387, 31373.0117224489)
        self.assertEqual(result, mock_response["results"])

#{'GeocodeInfo': [{'BUILDINGNAME': 'KAMPONG UBI VIEW', 'BLOCK': '351', 'ROAD': 'UBI AVENUE 1', 'POSTALCODE': '400351', 'XCOORD': '35501.9607216', 'YCOORD': '34191.1578935', 'LATITUDE': '1.325486284730739', 'LONGITUDE': '103.90072773995409'}, {'BUILDINGNAME': 'KAMPONG UBI VIEW', 'BLOCK': '352', 'ROAD': 'UBI AVENUE 1', 'POSTALCODE': '400352', 'XCOORD': '35354.2225417', 'YCOORD': '34162.4855169', 'LATITUDE': '1.3252270180708603', 'LONGITUDE': '103.89940022649942'}, {'BUILDINGNAME': 'KAMPONG UBI VIEW', 'BLOCK': '349', 'ROAD': 'UBI AVENUE 1', 'POSTALCODE': '400349', 'XCOORD': '35485.6405577', 'YCOORD': '34228.9805541', 'LATITUDE': '1.3258283432645641', 'LONGITUDE': '103.90058110378057'}]}

if __name__ == "__main__":
    unittest.main()



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name = 'pyonemap',
    version = '0.6.0',
    packages = find_packages(),
    install_requires = [
        'requests',
    ],
    author = 'Teo Cheng Guan',
    author_email = 'chengguan.teo@gmail.com',
    description = 'A Python package for interacting with OneMap API',
    long_description=long_description,  # Read from README.md
    long_description_content_type="text/markdown",
    url = 'https://github.com/chengguan/pyonemap',
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)



__author__ = "Teo Cheng Guan"
__copyright__ = "Copyright (C) 2023 Teo Cheng Guan"
__license__ = "MIT"
__version__ = "1.0"

import requests
import os

class OneMap:
    def __init__(self, api_key=None):
        self.base_url = "https://www.onemap.gov.sg"
        self.api_key = api_key

    def getToken(self, email, password):
        url = f"{self.base_url}/api/auth/post/getToken"
        payload = {"email": email, "password": password}

        try:
            response = requests.request("POST", url, json=payload)
            response.raise_for_status()  # Check for HTTP errors
        except requests.exceptions.RequestException as e:
            # Handle any exceptions that occurred during the request
            print(f"An error occurred: {e}")
            return None
        
        return response.json()

    def _make_request(self, endpoint, params):
        url = f"{self.base_url}{endpoint}"
        headers = {}

        if self.api_key:
            headers["Authorization"] = f"Bearer {self.api_key}"

        response = requests.get(url, params=params, headers=headers)

        if response.status_code == 200:
            return response.json()
        else:
            response.raise_for_status()

    '''
        searchVal string REQUIRED
        Keywords entered by users to filter the results

        returnGeom string REQUIRED
        Values: Y, N . Enter Y if user wants the geometry value returned.

        getAddrDetails string REQUIRED
        Values: Y, N . Enter Y if user wants address details returned.

        pageNum integer Optional. 
        Specifies the page to retrieve search results.

    '''
    def search(self, searchVal, returnGeom='Y', getAddressDetails='Y', pageNum=1):
        endpoint = f"/api/common/elastic/search"
        params = {"searchVal": searchVal, 
                  "returnGeom": returnGeom, 
                  "getAddrDetails": getAddressDetails, 
                  "pageNum": pageNum}

        response = self._make_request(endpoint, params)
        return response.get("results", [])

    '''
        location string
        Latitude and Longitude Coordinates in WGS84 format.

        buffer string Optional. 
        Values: 0-500 (in meters). Rounds up all buildings in a circumference from a point and search building add

        addressType string Optional. 
        Values: HDB or All. Allows users to define property types within the buffer/radius. If HDB is selected, result

        otherFeatures string Optional. 
        Values: Y or N. Allows uses the page to retrieve information on reservoirs, playgrounds, jetties, etc. Default is N.
    '''
    def revGeoCodeXy(self, x, y, buffer=40, addressType='All', otherFeatures='N'):
        endpoint = f"/api/public/revgeocodexy"

        params = {"location": f'{x},{y}', 
                  "buffer": buffer, 
                  "addressType": addressType, 
                  "otherFeatures": otherFeatures}

        response = self._make_request(endpoint, params)
        
        return response

    '''
        location string
        Latitude and Longitude Coordinates in WGS84 format.

        buffer string Optional. 
        Values: 0-500 (in meters). Rounds up all buildings in a circumference from a point and search building add

        addressType string Optional. 
        Values: HDB or All. Allows users to define property types within the buffer/radius. If HDB is selected, result

        otherFeatures string Optional. 
        Values: Y or N. Allows uses the page to retrieve information on reservoirs, playgrounds, jetties, etc. Default is N.
    '''
    def revGeoCode(self, lat, lon, buffer=40, addressType='All', otherFeatures='N'):
        endpoint = f"/api/public/revgeocode"

        params = {"location": f'{lat},{lon}', 
                  "buffer": buffer, 
                  "addressType": addressType, 
                  "otherFeatures": otherFeatures}

        response = self._make_request(endpoint, params)
        
        return response


if __name__ == "__main__":
    onemap = OneMap()
    
    location = onemap.search("5 Havelock Road", 'Y', 'Y')
    print(location)

    geocode = onemap.revGeoCodeXy(24291.97788882387, 31373.0117224489)
    print(geocode)

    geocode = onemap.revGeoCode(1.3254295, 103.9005321)
    print(geocode)